@@ -1,6 +1,5 @@
 from collections import namedtuple
 from datetime import datetime
-from typing import Dict
 from typing import Optional
 from typing import Type
 
@@ -61,7 +60,7 @@ class UserPartial(UserBase):
         yield "name", self.name
         yield "status", self.status
         yield "title", self.title
-        yield "join_date", self.join_date.timetuple()
+        yield "join_date", self.join_date
         yield "user_icon_url", self.user_icon_url
 
     def parse(self, user_tag: Tag = None):
@@ -89,8 +88,8 @@ class User(UserBase):
         self.join_date: datetime = datetime.fromtimestamp(0)
         self.profile: str = ""
         self.stats: UserStats = UserStats(0, 0, 0, 0, 0, 0)
-        self.info: Dict[str, str] = {}
-        self.contacts: Dict[str, str] = {}
+        self.info: dict[str, str] = {}
+        self.contacts: dict[str, str] = {}
         self.user_icon_url: str = ""
 
         self.parse()
@@ -99,7 +98,7 @@ class User(UserBase):
         yield "name", self.name
         yield "status", self.status
         yield "title", self.title
-        yield "join_date", self.join_date.timetuple()
+        yield "join_date", self.join_date
         yield "profile", self.profile
         yield "stats", self.stats._asdict()
         yield "info", self.info
@@ -122,6 +121,6 @@ class User(UserBase):
         self.title: str = parsed["title"]
         self.join_date: datetime = parsed["join_date"]
         self.stats: UserStats = UserStats(*parsed["stats"])
-        self.info: Dict[str, str] = parsed["info"]
-        self.contacts: Dict[str, str] = parsed["contacts"]
+        self.info: dict[str, str] = parsed["info"]
+        self.contacts: dict[str, str] = parsed["contacts"]
         self.user_icon_url: str = parsed["user_icon_url"]
